//----------------------------------------------------------------------------
//  Project Test
//  Power Softwares
//  Copyright  2009. All Rights Reserved.
//
//  SUBSYSTEM:    Test Application
//  FILE:         testapp.cpp
//  AUTHOR:       Jahanshahi, Mohsen
//
//  OVERVIEW
//  ~~~~~~~~
//  Source file for implementation of TTestApp (TApplication).
//
//----------------------------------------------------------------------------

#include <owl/pch.h>

#include <owl/buttonga.h>
#include <owl/docmanag.h>
#include <owl/statusba.h>
#include <owl/filedoc.h>
#include <owl/timegadg.h>

#include <classlib/cmdline.h>
#include <classlib/filename.h>

#include "testapp.h"
#include "tstedtvw.h"                        // Definition of client class.
#include "tstabtdl.h"                        // Definition of about dialog.


//{{TTestApp Implementation}}



//{{DOC_VIEW}}
DEFINE_DOC_TEMPLATE_CLASS(TFileDocument, TTestEditView, DocType1);
//{{DOC_VIEW_END}}

//{{DOC_MANAGER}}
DocType1 __dvt1("All Files", "*.*", 0, "txt", dtAutoDelete | dtUpdateDir | dtOverwritePrompt);
//{{DOC_MANAGER_END}}


//
// Build a response table for all messages/commands handled by the application.
//
DEFINE_RESPONSE_TABLE1(TSDIDecFrame, TDecoratedFrame)
//{{TSDIDecFrameRSP_TBL_BEGIN}}
  EV_WM_INITMENU,
  EV_WM_INITMENUPOPUP,
  EV_WM_DRAWITEM,
  EV_WM_MEASUREITEM,
  EV_WM_MENUCHAR,
  EV_MESSAGE(WM_UNINITMENUPOPUP, EvUnInitMenuPopup),
  EV_COMMAND(CM_DRAWRECESSED, CmDrawRecessed),
  EV_COMMAND(CM_ACCINTOOLTIP, CmAccInTooltip),
  EV_COMMAND(CM_SHOW_LINE,    CmShowLine),
  EV_COMMAND(CM_SHOW_XP,      CmShowXP),
  EV_COMMAND(CM_VIEW50,       CmView50),
  EV_COMMAND(CM_VIEW100,      CmView100),
  EV_COMMAND(CM_VIEW200,      CmView200),
  EV_COMMAND(CM_TOOLBAR,      CmToolBar),
  EV_COMMAND(CM_STATUSBAR,    CmStatusBar),
  EV_COMMAND(CM_BITMAPONOFF,  CmBitmapOnOff),
  EV_COMMAND(CM_CHANGETEXT,   CmChangeText),
  EV_COMMAND(CM_INSERTITEM,   CmInsertItem),
  EV_COMMAND(CM_REMOVEITEM,   CmRemoveItem),
  EV_COMMAND(CM_TEST_ITEM1,   CmTestItem1),
  EV_COMMAND(CM_TEST_ITEM2,   CmTestItem2),
  EV_COMMAND_ENABLE(CM_DRAWRECESSED, CeDrawRecessed),
  EV_COMMAND_ENABLE(CM_ACCINTOOLTIP, CeAccInTooltip),
  EV_COMMAND_ENABLE(CM_SHOW_LINE, CeShowLine),
  EV_COMMAND_ENABLE(CM_SHOW_XP, CeShowXP),
  EV_COMMAND_ENABLE(CM_VIEW100, CeView),
  EV_COMMAND_ENABLE(CM_VIEW200, CeView),
  EV_COMMAND_ENABLE(CM_VIEW50, CeView),
  EV_COMMAND_ENABLE(CM_TOOLBAR, CeToolbar),
  EV_COMMAND_ENABLE(CM_STATUSBAR, CeStatusbar),
  EV_COMMAND_ENABLE(CM_BITMAPONOFF, CeBitmapOnOff),
  EV_COMMAND_ENABLE(CM_CHANGETEXT, CeChangeText),
  EV_COMMAND_ENABLE(CM_INSERTITEM, CeInsertItem),
//{{TSDIDecFrameRSP_TBL_END}}
END_RESPONSE_TABLE;


//{{TSDIDecFrame Implementation}}


TSDIDecFrame::TSDIDecFrame(TWindow* parent, const char far* title, TWindow* clientWnd, bool trackMenuSelection, TModule* module)
:
  TDecoratedFrame(parent, title, clientWnd, trackMenuSelection, module),
  bBitmapOnOff(true),
  bStatusBar(true),
  bToolBar(true),
  iCommand(0),
  iView(1)
{
  // INSERT>> Your constructor code here.
  bmpMenu = new TBmpMenu(this);

  popupMenu = new TMenu(::Module->GetInstance(), TResId(IDM_TEST_MENU));
}

TSDIDecFrame::~TSDIDecFrame()
{
  // INSERT>> Your destructor code here.
  delete bmpMenu;

  delete popupMenu;
}

void TSDIDecFrame::SetupWindow()
{
  TDecoratedFrame::SetupWindow();

  HMENU hMenu = GetMenu();
  if (bmpMenu && hMenu)
    bmpMenu->RemapMenu(hMenu);
}

int TSDIDecFrame::GetHintText(uint id, char* buf, int size, THintText hintType) const
{
  if (bmpMenu) {
    int result = bmpMenu->PreGetHintText(GetMenu(), id, buf, size, hintType);

    if (result)
      return result;
  }

  return TDecoratedFrame::GetHintText(id, buf, size, hintType);
}

bool TSDIDecFrame::SetMenu(HMENU hMenu)
{
  if (bmpMenu && hMenu)
    bmpMenu->RemapMenu(hMenu);

  return TDecoratedFrame::SetMenu(hMenu);
}

void TSDIDecFrame::EvInitMenu(HMENU hMenu)
{
  if (bmpMenu)
    bmpMenu->RemapMenu(hMenu);

  TDecoratedFrame::EvInitMenu(hMenu);
}

void TSDIDecFrame::EvInitMenuPopup(HMENU hPopupMenu, uint index, bool sysMenu)
{
  if (bmpMenu)
    bmpMenu->EvInitMenuPopup(hPopupMenu, index, sysMenu);

  TDecoratedFrame::EvInitMenuPopup(hPopupMenu, index, sysMenu);
}

TResult TSDIDecFrame::EvUnInitMenuPopup(TParam1 param1, TParam2 param2)
{
  if (bmpMenu)
    bmpMenu->EvUnInitMenuPopup((HMENU)param1);

  return TSDIDecFrame::DefWindowProc(WM_UNINITMENUPOPUP, param1, param2);
}

void TSDIDecFrame::EvDrawItem(uint ctrlId, DRAWITEMSTRUCT far& drawInfo)
{
  if (bmpMenu && bmpMenu->DrawItem(drawInfo))
    return;

  TDecoratedFrame::EvDrawItem(ctrlId, drawInfo);
}

void TSDIDecFrame::EvMeasureItem(uint ctrlId, MEASUREITEMSTRUCT far& measureInfo)
{
  if (bmpMenu && bmpMenu->MeasureItem(measureInfo))
    return;

  TDecoratedFrame::EvMeasureItem(ctrlId, measureInfo);
}

int32 TSDIDecFrame::EvMenuChar(uint nChar, uint menuType, HMENU hMenu)
{
  int32 result;

  if (bmpMenu && bmpMenu->FindKeyboardShortcut(nChar, menuType, hMenu, result))
    return result;

  return TDecoratedFrame::EvMenuChar(nChar, menuType, hMenu);
}

void TSDIDecFrame::CmDrawRecessed()
{
  bmpMenu->DrawRecessed(!bmpMenu->DrawRecessed());
}

void TSDIDecFrame::CmAccInTooltip()
{
  bmpMenu->ShowAcclInTooltips(!bmpMenu->ShowAcclInTooltips());
}

void TSDIDecFrame::CmShowLine()
{
  bmpMenu->ShowLine(!bmpMenu->ShowLine());
}

void TSDIDecFrame::CmShowXP()
{
  bmpMenu->XPStyle(!bmpMenu->XPStyle());
}

void TSDIDecFrame::CmView50()
{
  iView = 0;
}

void TSDIDecFrame::CmView100()
{
  iView = 0;
}

void TSDIDecFrame::CmView200()
{
  iView = 2;
}

void TSDIDecFrame::CmToolBar()
{
  bToolBar = !bToolBar;
}

void TSDIDecFrame::CmStatusBar()
{
  bStatusBar = !bStatusBar;
}

void TSDIDecFrame::CmBitmapOnOff()
{
  bBitmapOnOff = !bBitmapOnOff;
}

void TSDIDecFrame::CmChangeText()
{
  iCommand++;
}

void TSDIDecFrame::CmInsertItem()
{
  HMENU hMenu = GetMenu();

  if (hMenu) {
    hMenu = GetSubMenu(hMenu, 3);

    if (hMenu) {
      char buffer[256];

      if (popupMenu && popupMenu->GetMenuString(0, buffer, sizeof(buffer),
                                                MF_BYPOSITION)) {
        ::AppendMenu(hMenu, MF_POPUP | MF_STRING,
                     (UINT)popupMenu->GetSubMenu(0), buffer);
        ::SetMenuDefaultItem(popupMenu->GetSubMenu(0), CM_TEST_ITEM2, false);
      }

      bmpMenu->RemapMenu(hMenu, true);
    }
  }
}

void TSDIDecFrame::CmRemoveItem()
{
  HMENU hMenu = GetMenu();

  if (hMenu) {
    hMenu = GetSubMenu(hMenu, 3);

    if (hMenu) {
      int count = ::GetMenuItemCount(hMenu);

      if (count == 15) {
        ::RemoveMenu(hMenu, count - 1, MF_BYPOSITION);

        bmpMenu->RemapMenu(hMenu, true);
      }
    }
  }
}

void TSDIDecFrame::CmTestItem1()
{
  MessageBox("Test Item with Bitmap", "Test", MB_OK);
}

void TSDIDecFrame::CmTestItem2()
{
  MessageBox("Default Item", "Test", MB_OK);
}

void TSDIDecFrame::CeDrawRecessed(TCommandEnabler &tce)
{
  tce.SetCheck(bmpMenu->DrawRecessed());
}

void TSDIDecFrame::CeAccInTooltip(TCommandEnabler &tce)
{
  tce.SetCheck(bmpMenu->ShowAcclInTooltips());
}

void TSDIDecFrame::CeShowLine(TCommandEnabler &tce)
{
  tce.SetCheck(bmpMenu->ShowLine());
}

void TSDIDecFrame::CeShowXP(TCommandEnabler &tce)
{
  tce.SetCheck(bmpMenu->XPStyle());
}

void TSDIDecFrame::CeView(TCommandEnabler &tce)
{
  TMenuItemEnabler* tme = TYPESAFE_DOWNCAST(&tce, TMenuItemEnabler);
  if (tme && tme->GetMenu()) {
    if (::CheckMenuRadioItem(tme->GetMenu(), CM_VIEW50, CM_VIEW200,
                         CM_VIEW50 + iView, MF_BYCOMMAND) == false)
      MessageBeep(0);

  }
}

void TSDIDecFrame::CeToolbar(TCommandEnabler &tce)
{
  tce.SetCheck(bToolBar);
}

void TSDIDecFrame::CeStatusbar(TCommandEnabler &tce)
{
  tce.SetCheck(bStatusBar);
}

void TSDIDecFrame::CeBitmapOnOff(TCommandEnabler& tce)
{
  tce.SetCheck(bBitmapOnOff);
}

void TSDIDecFrame::CeChangeText(TCommandEnabler &tce)
{
  char buffer[256];
  wsprintf(buffer, "&Change Text: command %d", iCommand);

  TMenuItemEnabler* tme = TYPESAFE_DOWNCAST(&tce, TMenuItemEnabler);
  if (tme && tme->GetMenu());
    bmpMenu->SetMenuItemText(tme->GetMenu(), tme->GetPosition(), buffer);
}

void TSDIDecFrame::CeInsertItem(TCommandEnabler& tce)
{
  TMenuItemEnabler* tme = TYPESAFE_DOWNCAST(&tce, TMenuItemEnabler);
  if (tme && tme->GetMenu())
    tme->Enable(::GetMenuItemCount(tme->GetMenu()) < 15);
}

//
// Build a response table for all messages/commands handled by the application.
//
DEFINE_RESPONSE_TABLE2(TTestApp, TRecentFiles, TApplication)
//{{TTestAppRSP_TBL_BEGIN}}
  EV_OWLVIEW(dnCreate, EvNewView),
  EV_OWLVIEW(dnClose,  EvCloseView),
  EV_COMMAND(CM_FILESEND, CmFileSend),
  EV_COMMAND_ENABLE(CM_FILESEND, CeFileSend),
  EV_COMMAND(CM_HELPABOUT, CmHelpAbout),
  EV_COMMAND(CM_FILEPRINT, CmFilePrint),
  EV_COMMAND(CM_FILEPRINTERSETUP, CmFilePrintSetup),
  EV_COMMAND(CM_FILEPRINTPREVIEW, CmFilePrintPreview),
  EV_COMMAND_ENABLE(CM_FILEPRINT, CmPrintEnable),
  EV_COMMAND_ENABLE(CM_FILEPRINTERSETUP, CmPrintEnable),
  EV_COMMAND_ENABLE(CM_FILEPRINTPREVIEW, CmPrintEnable),
  EV_WM_DROPFILES,
  EV_WM_WININICHANGE,
  EV_OWLDOCUMENT(dnCreate, EvOwlDocument),
  EV_OWLDOCUMENT(dnRename, EvOwlDocument),
  EV_REGISTERED(MruFileMessage, CmFileSelected),
  EV_COMMAND_ENABLE(CM_EXIT, CeExit),
//{{TTestAppRSP_TBL_END}}
END_RESPONSE_TABLE;


//--------------------------------------------------------
// TTestApp
// ~~~~~
//
TTestApp::TTestApp()
:
 TApplication("Test"),
 TRecentFiles(".\\Test.ini", 4)
{
  Printer = 0;
  Printing = 0;

  SetDocManager(new TDocManager(dmSDI, this));

  ApxMail = new TMailer();

  // INSERT>> Your constructor code here.
}


TTestApp::~TTestApp()
{
  delete Printer;

  delete ApxMail;

  // INSERT>> Your destructor code here.
}


void TTestApp::CreateGadgets(TDockableControlBar* cb, bool server)
{
  if (!server) {
    cb->Insert(*new TButtonGadget(CM_FILENEW, CM_FILENEW));
    cb->Insert(*new TButtonGadget(CM_FILEOPEN, CM_FILEOPEN));
    cb->Insert(*new TButtonGadget(CM_FILESAVE, CM_FILESAVE));
    cb->Insert(*new TSeparatorGadget(6));
  }

  cb->Insert(*new TButtonGadget(CM_EDITCUT, CM_EDITCUT));
  cb->Insert(*new TButtonGadget(CM_EDITCOPY, CM_EDITCOPY));
  cb->Insert(*new TButtonGadget(CM_EDITPASTE, CM_EDITPASTE));
  cb->Insert(*new TSeparatorGadget(6));
  cb->Insert(*new TButtonGadget(CM_EDITUNDO, CM_EDITUNDO));
  cb->Insert(*new TSeparatorGadget(6));
  cb->Insert(*new TButtonGadget(CM_EDITFIND, CM_EDITFIND));
  cb->Insert(*new TButtonGadget(CM_EDITFINDNEXT, CM_EDITFINDNEXT));

  if (!server) {
    cb->Insert(*new TSeparatorGadget(6));
    cb->Insert(*new TButtonGadget(CM_FILEPRINT, CM_FILEPRINT));
    cb->Insert(*new TButtonGadget(CM_FILEPRINTPREVIEW, CM_FILEPRINTPREVIEW));
  }

  // Add caption and fly-over help hints.
  //
  cb->SetCaption("Toolbar");
  cb->SetHintMode(TGadgetWindow::EnterHints);
}


void TTestApp::SetupSpeedBar(TDecoratedFrame* frame)
{
  ApxHarbor = new THarbor(*frame);

  // Create default toolbar New and associate toolbar buttons with commands.
  //
  TDockableControlBar* cb = new TDockableControlBar(frame);
  CreateGadgets(cb);

  // Setup the toolbar ID used by OLE 2 for toolbar negotiation.
  //
  cb->Attr.Id = IDW_TOOLBAR;

  ApxHarbor->Insert(*cb, alTop);
}


//--------------------------------------------------------
// TTestApp
// ~~~~~
// Application main window construction & intialization.
//
void TTestApp::InitMainWindow()
{
  if (nCmdShow != SW_HIDE)
    nCmdShow = (nCmdShow != SW_SHOWMINNOACTIVE) ? SW_SHOWNORMAL : nCmdShow;

  TSDIDecFrame* frame = new TSDIDecFrame(0, GetName(), 0, true, this);

  // Enable acceptance of dropped files
  //
  frame->Attr.ExStyle |= WS_EX_ACCEPTFILES;

  // Assign icons for this application.
  //
  frame->SetIcon(this, IDI_SDIAPPLICATION);
  frame->SetIconSm(this, IDI_SDIAPPLICATION);

  // Menu associated with window and accelerator table associated with table.
  //
  frame->AssignMenu(IDM_SDI);

  // Associate with the accelerator table.
  //
  frame->Attr.AccelTable = IDM_SDI;

  TStatusBar* sb = new TStatusBar(frame, TGadget::WndRecessed,
                                  TStatusBar::CapsLock |
                                  TStatusBar::NumLock  |
                                  TStatusBar::Overtype |
                                  TStatusBar::ScrollLock,
                                  new TGadgetWindowFont(8));

  sb->Insert(*new TTimeGadget(&TTimeGadget::GetLocalTime, 0,
                              TGadget::WndRecessed, TTextGadget::Center, 6));
  sb->SetWideHints(false);

  frame->Insert(*sb, TDecoratedFrame::Bottom);

  SetupSpeedBar(frame);

  SetMainWindow(frame);

  frame->SetMenuDescr(TMenuDescr(IDM_SDI));

}


//--------------------------------------------------------
// TTestApp
// ~~~~~
// Application instance initialization.
//
void TTestApp::InitInstance()
{
  TApplication::InitInstance();
  GetDocManager()->CreateAnyDoc(0, dtNewDoc);
  ProcessCmdLine(lpCmdLine);
}


//--------------------------------------------------------
// TTestApp
// ~~~~~
// Process command line parameters.
//
void TTestApp::ProcessCmdLine(char * CmdLine)
{
  TCmdLine cmd(CmdLine);

  while (cmd.Kind != TCmdLine::Done) {
    if (cmd.Kind == TCmdLine::Option) {
      if (strnicmp(cmd.Token, "unregister", cmd.TokenLen) == 0) {
        UnRegisterInfo();
        return;
      }
    }
    cmd.NextToken();
  }

  RegisterInfo();
}


//--------------------------------------------------------
// TTestApp
// ~~~~~
// Register application info.
//
void TTestApp::RegisterInfo()
{
  TAPointer<char> buffer = new char[_MAX_PATH];

  GetModuleFileName(buffer, _MAX_PATH);

  TRegKey(TRegKey::ClassesRoot, "Test.Application\\DefaultIcon").SetDefValue(0, REG_SZ, buffer, strlen(buffer));
  strcat(buffer, ",1");
  TRegKey(TRegKey::ClassesRoot, "Test.Document.1\\DefaultIcon").SetDefValue(0, REG_SZ, buffer, strlen(buffer));
  strcpy(buffer, "Test.Document.1");
  TRegKey(TRegKey::ClassesRoot, ".txt").SetDefValue(0, REG_SZ, buffer, strlen(buffer));
}


//--------------------------------------------------------
// TTestApp
// ~~~~~
// Unregister application info.
//
void TTestApp::UnRegisterInfo()
{
  TAPointer<char> buffer = new char[_MAX_PATH];

  GetModuleFileName(buffer, _MAX_PATH);

  TRegKey(TRegKey::ClassesRoot, "Test.Application").DeleteKey("DefaultIcon");
  TRegKey(TRegKey::ClassesRoot, "Test.Document.1").DeleteKey("DefaultIcon");

  TRegKey::ClassesRoot.DeleteKey("Test.Application");
  TRegKey::ClassesRoot.DeleteKey("Test.Document.1");
  TRegKey::ClassesRoot.DeleteKey(".txt");
}



//--------------------------------------------------------
// TTestApp
// ~~~~~
// Response Table handlers:
//
void TTestApp::EvNewView(TView& view)
{
  GetMainWindow()->SetClientWindow(view.GetWindow());
  if (!view.IsOK())
    GetMainWindow()->SetClientWindow(0);
  else if (view.GetViewMenu())
    GetMainWindow()->MergeMenu(*view.GetViewMenu());
}


void TTestApp::EvCloseView(TView&)
{
  GetMainWindow()->SetClientWindow(0);
  GetMainWindow()->SetCaption("Test");
}


//--------------------------------------------------------
// TTestApp
// ~~~~~~~~~~
// Menu File Print command
//
void TTestApp::CmFilePrint()
{
  // Create Printer object if not already created.
  //
  if (!Printer)
    Printer = new TPrinter(this);

  TDocument* currentDoc = GetDocManager()->GetCurrentDoc();

  TAPointer<char> docName = new char[_MAX_PATH];

  if (currentDoc->GetTitle())
    strcpy(docName, currentDoc->GetTitle());
  else
    strcpy(docName, "Document");

  // Create Printout window and set characteristics.
  //
  TApxPrintout printout(Printer, docName, GetMainWindow()->GetClientWindow());
  printout.SetBanding(true);

  Printing++;

  // Bring up the Print dialog and print the document.
  //
  Printer->Print(GetWindowPtr(GetActiveWindow()), printout, true);

  Printing--;
}


//--------------------------------------------------------
// TTestApp
// ~~~~~~~~~~
// Menu File Print Setup command
//
void TTestApp::CmFilePrintSetup()
{
  if (!Printer)
    Printer = new TPrinter(this);

  // Bring up the Print Setup dialog.
  //
  Printer->Setup(GetMainWindow());
}


//--------------------------------------------------------
// TTestApp
// ~~~~~~~~~~
// Menu File Print Preview command
//
void TTestApp::CmFilePrintPreview()
{
  TSDIDecFrame* sdiFrame = TYPESAFE_DOWNCAST(GetMainWindow(), TSDIDecFrame);
  if (sdiFrame) {
    if (!Printer)
      Printer = new TPrinter(this);

    Printing++;

    TApxPreviewWin* prevW = new TApxPreviewWin(sdiFrame, Printer, sdiFrame->GetClientWindow(), "Print Preview", new TLayoutWindow(0));
    prevW->Create();

    // Here we resize the preview window to take the size of the MainWindow, then
    // hide the MainWindow.
    //
    TFrameWindow * mainWindow =  GetMainWindow();
    TRect r = mainWindow->GetWindowRect();
    prevW->MoveWindow(r);
    prevW->ShowWindow(SW_SHOWNORMAL);
    mainWindow->ShowWindow(SW_HIDE);

    BeginModal(GetMainWindow());

    // After the user closes the preview Window, we take its size and use it
    // to size the MainWindow, then show the MainWindow again.
    //
    r = prevW->GetWindowRect();
    mainWindow->MoveWindow(r);
    mainWindow->ShowWindow(SW_SHOWNORMAL);

    Printing--;

    // Now that printing is off we can invalidate because the edit window to repaint.
    //
    GetMainWindow()->SetRedraw(true);
    GetMainWindow()->Invalidate();

    // We must destroy the preview window explicitly.  Otherwise, the window will
    // not be destroyed until it's parent the MainWindow is destroyed.
    //
    prevW->Destroy();
    delete prevW;
  }
}


//--------------------------------------------------------
// TTestApp
// ~~~~~~~~~~
// Menu enabler used by Print, Print Setup and Print Preview.
//
void TTestApp::CmPrintEnable(TCommandEnabler& tce)
{
  // If we have a Printer already created just test if all is okay.
  // Otherwise create a Printer object and make sure the printer really exists
  // and then delete the Printer object.
  //
  if (!Printer) {
    Printer = new TPrinter(this);
      tce.Enable(!Printer->GetSetup().Error);
  }
  else
    tce.Enable(!Printer->GetSetup().Error);
}


void TTestApp::CeFileSend(TCommandEnabler& ce)
{
  ce.Enable((GetDocManager()->GetCurrentDoc() != 0)
            && ApxMail->IsMAPIAvailable());
}


void TTestApp::CmFileSend ()
{
  // Check to see if a document exists
  //
  TDocument* currentDoc = GetDocManager()->GetCurrentDoc();

  if (currentDoc) {
    TAPointer<char> savedPath = new char[_MAX_PATH];
    TAPointer<char> docName = new char[_MAX_PATH];

    bool dirtyState = currentDoc->IsDirty();

    if (currentDoc->GetDocPath())
      strcpy(savedPath, currentDoc->GetDocPath());
    else
      strcpy(savedPath, "");

    if (currentDoc->GetTitle())
      strcpy(docName, currentDoc->GetTitle());
    else
      strcpy(docName, "Document");

    TFileName tempFile(TFileName::TempFile);

    currentDoc->SetDocPath(tempFile.Canonical().c_str());
    currentDoc->Commit(true);

    currentDoc->SetDocPath(savedPath);
    currentDoc->SetDirty(dirtyState);

    ApxMail->SendDocuments(GetMainWindow(), tempFile.Canonical().c_str(), docName, false);

    tempFile.Remove();
  }
}


//--------------------------------------------------------
// TTestApp
// ~~~~~~~~~~~
//
// Menu Help About Test command
void TTestApp::CmHelpAbout()
{
  // Show the modal dialog.
  //
  TTestAboutDlg(GetMainWindow()).Execute();
}


void TTestApp::EvDropFiles(TDropInfo drop)
{
  // Tell DragQueryFile the file interested in (0) and the length of your buffer.
  //
  int    nameLength = drop.DragQueryFileNameLen(0) + 1;
  char*  fileName = new char[nameLength];

  drop.DragQueryFile(0, fileName, nameLength);

  // Open the file that was dropped.
  //
  AddFile(fileName);

  // Release the memory allocated for this handle with DragFinish.
  //
  drop.DragFinish();
  delete[] fileName;
}


void TTestApp::AddFile(const char* FileName)
{
  TDocTemplate* tpl = GetDocManager()->MatchTemplate(FileName);
  if (tpl)
    GetDocManager()->CreateDoc(tpl, FileName);
}


void TTestApp::EvOwlDocument(TDocument& doc)
{
  if (doc.GetDocPath())
    SaveMenuChoice(doc.GetDocPath());
}


int32 TTestApp::CmFileSelected(uint wp, int32)
{
  TAPointer<char> text = new char[_MAX_PATH];

  GetMenuText(wp, text, _MAX_PATH);
  TDocTemplate* tpl = GetDocManager()->MatchTemplate(text);
  if (tpl)
    GetDocManager()->CreateDoc(tpl, text);
  return 0;
}

void TTestApp::EvWinIniChange(char far* section)
{
  if (strcmp(section, "windows") == 0) {
    // If the device changed in the WIN.INI file then the printer
    // might have changed.  If we have a TPrinter(Printer) then
    // check and make sure it's identical to the current device
    // entry in WIN.INI.
    //
    if (Printer) {
      const int bufferSize = 255;
      char printDBuffer[bufferSize];
      LPSTR printDevice = printDBuffer;
      LPSTR devName;
      LPSTR driverName = 0;
      LPSTR outputName = 0;
      if (::GetProfileString("windows", "device", "", printDevice, bufferSize)) {
        // The string which should come back is something like:
        //
        //      HP LaserJet III,hppcl5a,LPT1:
        //
        // Where the format is:
        //
        //      devName,driverName,outputName
        //
        devName = printDevice;
        while (*printDevice) {
          if (*printDevice == ',') {
            *printDevice++ = 0;
            if (!driverName)
              driverName = printDevice;
            else
              outputName = printDevice;
          }
          else
            printDevice = ::AnsiNext(printDevice);
        }

        if (Printer->GetSetup().Error != 0                ||
            strcmp(devName, Printer->GetSetup().GetDeviceName()) != 0  ||
            strcmp(driverName, Printer->GetSetup().GetDriverName()) != 0 ||
            strcmp(outputName, Printer->GetSetup().GetOutputName()) != 0 ) {
          // New printer installed so get the new printer device now.
          //
          delete Printer;
          Printer = new TPrinter(this);
        }
      }
      else {
        // No printer installed(GetProfileString failed).
        //
        delete Printer;
        Printer = new TPrinter(this);
      }
    }
  }
}

void TTestApp::CeExit(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  TRecentFiles::CeExit(tce);

  TFrameWindow* window = GetMainWindow();

  if (window) {
    HMENU hMenu = window->GetMenu();
    window->SetMenu(hMenu);
  }

}

int OwlMain(int , char* [])
{
  TTestApp   app;
  return app.Run();
}

