//----------------------------------------------------------------------------
//  Project Test
//  Power Softwares
//  Copyright  2009. All Rights Reserved.
//
//  SUBSYSTEM:    Test Application
//  FILE:         testapp.h
//  AUTHOR:       Jahanshahi, Mohsen
//
//  OVERVIEW
//  ~~~~~~~~
//  Class definition for TTestApp (TApplication).
//
//----------------------------------------------------------------------------
#if !defined(testapp_h)              // Sentry, use file only if it's not already included.
#define testapp_h

#include <owl/controlb.h>
#include <owl/docking.h>
#include <owl/mailer.h>
#include <owl/printer.h>
#include <owl/rcntfile.h>

#include <classlib/bags.h>


#include "apxprint.h"
#include "apxprev.h"
#include "bmpmenu.h"

#include "testapp.rh"            // Definition of all resources.


//
// FrameWindow must be derived to override Paint for Preview and Print.
//
//{{TDecoratedFrame = TSDIDecFrame}}
class TSDIDecFrame : public TDecoratedFrame {
  public:
    TSDIDecFrame(TWindow* parent, const char far* title, TWindow* clientWnd, bool trackMenuSelection = false, TModule* module = 0);
    ~TSDIDecFrame();

//{{TSDIDecFrameVIRTUAL_BEGIN}}
  public:
    virtual void SetupWindow();
    virtual int GetHintText(uint id, char* buf, int size, THintText hintType) const;
    virtual bool SetMenu(HMENU hMenu);
//{{TSDIDecFrameVIRTUAL_END}}

//{{TSDIDecFrameRSP_TBL_BEGIN}}
  protected:
    void EvInitMenu(HMENU hMenu);
    void EvInitMenuPopup(HMENU hPopupMenu, uint index, bool sysMenu);
    TResult EvUnInitMenuPopup(TParam1 param1, TParam2 param2);
    void EvDrawItem(uint ctrlId, DRAWITEMSTRUCT far& drawInfo);
    void EvMeasureItem(uint ctrlId, MEASUREITEMSTRUCT far& measureInfo);
    int32 EvMenuChar(uint nChar, uint menuType, HMENU hMenu);
    void CmDrawRecessed();
    void CmAccInTooltip();
    void CmShowLine();
    void CmShowXP();
    void CmView50();
    void CmView100();
    void CmView200();
    void CmToolBar();
    void CmStatusBar();
    void CmBitmapOnOff();
    void CmChangeText();
    void CmInsertItem();
    void CmRemoveItem();
    void CmTestItem1();
    void CmTestItem2();
    void CeAccInTooltip(TCommandEnabler &tce);
    void CeDrawRecessed(TCommandEnabler &tce);
    void CeShowLine(TCommandEnabler &tce);
    void CeShowXP(TCommandEnabler &tce);
    void CeView(TCommandEnabler &tce);
    void CeToolbar(TCommandEnabler &tce);
    void CeStatusbar(TCommandEnabler &tce);
    void CeBitmapOnOff(TCommandEnabler& tce);
    void CeChangeText(TCommandEnabler &tce);
    void CeInsertItem(TCommandEnabler &tce);
//{{TSDIDecFrameRSP_TBL_END}}

  protected:
    TBmpMenu* bmpMenu;

    TMenu* popupMenu;

    bool bBitmapOnOff;
    bool bStatusBar;
    bool bToolBar;

    int iCommand;
    int iView;

DECLARE_RESPONSE_TABLE(TSDIDecFrame);
};    //{{TSDIDecFrame}}

//{{TApplication = TTestApp}}
class TTestApp : public TApplication, public TRecentFiles {
  private:
    void SetupSpeedBar(TDecoratedFrame* frame);
    void AddFile(const char* fileName);
    void ProcessCmdLine(char * CmdLine);
    void RegisterInfo();
    void UnRegisterInfo();

  public:
    TTestApp();
    virtual ~TTestApp();

    void CreateGadgets(TDockableControlBar* cb, bool server = false);
    THarbor*        ApxHarbor;

    // Public data members used by the print menu commands and Paint routine in MDIChild.
    //
    TPrinter*       Printer;                            // Printer support.
    int             Printing;                           // Printing in progress.

    TMailer*        ApxMail;

//{{TTestAppVIRTUAL_BEGIN}}
  public:
    virtual void InitMainWindow();
    virtual void InitInstance();
//{{TTestAppVIRTUAL_END}}

//{{TTestAppRSP_TBL_BEGIN}}
  protected:
    void EvNewView(TView& view);
    void EvCloseView(TView& view);
    void CeFileSend(TCommandEnabler& ce);
    void CmFileSend();
    void CmHelpAbout();
    void CmFilePrint();
    void CmFilePrintSetup();
    void CmFilePrintPreview();
    void CmPrintEnable(TCommandEnabler& tce);
    void EvDropFiles(TDropInfo drop);
    void EvWinIniChange(char far* section);
    void EvOwlDocument(TDocument& doc);
    int32 CmFileSelected(uint wp, int32 lp);
    void CeExit(TCommandEnabler &tce);
//{{TTestAppRSP_TBL_END}}
DECLARE_RESPONSE_TABLE(TTestApp);
};    //{{TTestApp}}


#endif  // testapp_h sentry.
