// ****************************************************************************
// Copyright (C) 1998 by Dieter Windau
// All rights reserved
//
// BmpMenu.cpp: implementation file
// Version:     1.2
// Date:        07.08.1998
// Author:      Dieter Windau
//
// TBmpMenu is a freeware OWL class that shows bitmaps
// on the left side of menus like MS Office 97
//
// Portions of code are based on MFC class CMenuSpawn written by Iuri
// Apollonio and BCMenu written by Brent Corkum. Very special thanks.
//
// You are free to use/modify this code but leave this header intact.
// May not be sold for profit.
//
// Tested with Borland C++ 5.02, OWL 5.02 under Windows NT 4.0 SP3 but I think
// the class would work with Windows 95 too.
// This file is provided "as is" with no expressed or implied warranty.
// Use at your own risk.
//
// Please send me bug reports, bug fixes, enhancements, requests, etc., and
// I'll try to keep this in next versions:
//   EMail: dieter.windau@usa.net
//   Web:   http://www.members.aol.com/SoftEngage
// ****************************************************************************

// 28/02/2002  Luigi Bianchi made some changes to make this code
// work without BIDS and started to add Office XP Look and Feel
// CHeck updates at www.luigibianchi.com/owl
// 02/03/2002	Luigi Bianchi, change the way in which XPExtraWidth
// was computed due to 1 pixel error when using 20 * 20 bitmaps

// ****************************************************************************
// 4/05/2009 Mohsen Jahanshahi added still more feel and look of windows XP
// With appology to Luigi Bianchi for using BIDS in this code
//

#include <owl/gdiobjec.h>
#include <owl/menu.h>
#include <owl/module.h>

#include <stdlib.h>
#include <string.h>

#include "bmpmenu.h"

#define DEFAULT_BITMAP_WIDTH          19
#define DEFAULT_BITMAP_HEIGHT         18

#define BITMAP_EXTRA_WIDTH            1
#define XP_EXTRA_WIDTH                4

#define MAX_STRING_LENGTH             256

typedef DWORD HLSCOLOR;

#define HLS(h,l,s) ((HLSCOLOR)(((BYTE)(h)|((WORD)((BYTE)(l))<<8))|(((DWORD)(BYTE)(s))<<16)))

#define HLS_H(hls) ((BYTE)(hls))
#define HLS_L(hls) ((BYTE)(((WORD)(hls))>>8))
#define HLS_S(hls) ((BYTE)((hls)>>16))

// Helper functions
//
COLORREF LightenColor(COLORREF color, BYTE value)
{
  BYTE red   = GetRValue(color);
  BYTE green = GetGValue(color);
  BYTE blue  = GetBValue(color);

  red   = (BYTE)min(red   + value, 255);
  green = (BYTE)min(green + value, 255);
  blue  = (BYTE)min(blue  + value, 255);

  return RGB(red, green, blue);
}

COLORREF DarkenColor(COLORREF color, BYTE value)
{
  BYTE red   = GetRValue(color);
  BYTE green = GetGValue(color);
  BYTE blue  = GetBValue(color);

  red   = (BYTE)(red   >= value ? red   - value : 0);
  green = (BYTE)(green >= value ? green - value : 0);
  blue  = (BYTE)(blue  >= value ? blue  - value : 0);

  return RGB(red, green, blue);
}

COLORREF MergeColors(COLORREF color1, COLORREF color2)
{
  BYTE red   = (BYTE)((GetRValue(color1) + GetRValue(color2)) / 2);
  BYTE green = (BYTE)((GetGValue(color1) + GetGValue(color2)) / 2);
  BYTE blue  = (BYTE)((GetBValue(color1) + GetBValue(color2)) / 2);

  return RGB(red, green, blue);
}

void FillSolidRect(HDC hdc, int x, int y, int cx, int cy, COLORREF color)
{
  if (!hdc)
    return;

  ::SetBkColor(hdc, color);

  TRect rect(x, y, x + cx, y + cy);
  ::ExtTextOut(hdc, 0, 0, ETO_OPAQUE, &rect, 0, 0, 0);
}

void FillSolidRect(HDC hdc, const TRect& rect, COLORREF color)
{
  if (!hdc)
    return;

  ::SetBkColor(hdc, color);
  ::ExtTextOut(hdc, 0, 0, ETO_OPAQUE, &rect, 0, 0, 0);
}

void Draw3dRect(HDC hdc, int x, int y, int cx, int cy,
                COLORREF topLeftColor, COLORREF bottomRightColor)
{
  if (!hdc)
    return;

  FillSolidRect(hdc, x,      y,      cx - 1,  1,      topLeftColor);
  FillSolidRect(hdc, x,      y,      1,       cy - 1, topLeftColor);
  FillSolidRect(hdc, x + cx, y,     -1,       cy,     bottomRightColor);
  FillSolidRect(hdc, x,      y + cy, cx,     -1,      bottomRightColor);
}

void Draw3dRect(HDC hdc, const TRect& rect,
                COLORREF topLeftColor, COLORREF bottomRightColor)
{
  if (!hdc)
    return;

  Draw3dRect(hdc, rect.left, rect.top, rect.Width(), rect.Height(),
             topLeftColor, bottomRightColor);
}

HLSCOLOR RGB2HLS(COLORREF color)
{
  BYTE red   = GetRValue(color);
  BYTE green = GetGValue(color);
  BYTE blue  = GetBValue(color);

  unsigned char minval = min(red, min(green, blue));
  unsigned char maxval = max(red, max(green, blue));

  float dif = maxval - minval;
  float sum = maxval + minval;

  float luminance  = sum / 510.0;
  float saturation = 0.0;
  float hue        = 0.0;

  if (maxval != minval) {
    float rnorm = (maxval - red)   / dif;
    float gnorm = (maxval - green) / dif;
    float bnorm = (maxval - blue)  / dif;

    saturation = (luminance <= 0.5) ? dif / sum : dif / (510.0 - sum);

    if (red == maxval)
      hue = 60.0 * (6.0 + bnorm - gnorm);

    if (green == maxval)
      hue = 60.0 * (2.0 + rnorm - bnorm);

    if (blue == maxval)
      hue = 60.0 * (4.0 + gnorm - rnorm);

    if (hue > 360.0)
      hue -= 360.0;
  }

  return HLS((hue * 255) / 360, luminance * 255, saturation * 255);
}

static BYTE _ToRGB(float rm1, float rm2, float rh)
{
  if (rh > 360.0)
    rh -= 360.0;
  else if (rh < 0.0)
    rh += 360.0;

  if (rh < 60.0)
    rm1 += rh * (rm2 - rm1) / 60.0;
  else if (rh < 180.0)
    rm1 = rm2;
  else if (rh < 240.0)
    rm1 += (240.0 - rh) * (rm2 - rm1) / 60.0;

  return (BYTE)(rm1 * 255);
}

COLORREF HLS2RGB(HLSCOLOR hls)
{
  BYTE h = HLS_H(hls);
  BYTE l = HLS_L(hls);
  BYTE s = HLS_S(hls);

  float hue        = h * 360 / 255.0;
  float luminance  = l       / 255.0;
  float saturation = s       / 255.0;

  if (saturation == 0.0)
    return RGB(l, l, l);

  float rm1, rm2;

  if (luminance <= 0.5)
    rm2 =              luminance + luminance * saturation;
  else
    rm2 = saturation + luminance - luminance * saturation;

  rm1 = 2.0 * luminance - rm2;

  BYTE red   = _ToRGB(rm1, rm2, hue + 120.0);
  BYTE green = _ToRGB(rm1, rm2, hue);
  BYTE blue  = _ToRGB(rm1, rm2, hue - 120.0);

  return RGB(red, green, blue);
}

COLORREF HLS_TRANSFORM(COLORREF rgb, int percent_L, int percent_S)
{
  HLSCOLOR hls = RGB2HLS(rgb);

  BYTE h = HLS_H(hls);
  BYTE l = HLS_L(hls);
  BYTE s = HLS_S(hls);

  if (percent_L > 0)
    l = BYTE(l + ((255 - l) * percent_L) / 100);
  else if (percent_L < 0)
    l = BYTE((l * (100 + percent_L)) / 100);

  if (percent_S > 0)
    s = BYTE(s + ((255 - s) * percent_S) / 100);
  else if (percent_S < 0)
    s = BYTE((s * (100 + percent_S)) / 100);

  return HLS2RGB(HLS(h, l, s));
}

// TBmpMenuItem implementation
//
int
TBmpMenuItem::GetText(char* buffer, int size, bool bNoAccel) const
{
  int n = 0;

  char* ptr = text;

  while (*ptr != 0 && n < size) {
    if (!bNoAccel || *ptr != '&') {
      *buffer = *ptr;

      buffer++;
      n++;
    }

    ptr++;
  }

  if (n < size)
    *buffer = 0;

  return n;
}

void
TBmpMenuItem::SetText(const char* txt)
{
  if (text) {
    delete[] text;
    text = new char[strlen(txt) + 1];
  }

  strcpy(text, txt);
}

// TBmpMenu implementation
//
TBmpMenu::TBmpMenu(TWindow* pWindow, TModule* pModule)
:
 parent(pWindow),
 MSImage32(TMSImage32Module::GetInstance()),
 bDrawRecessed(true), bShowAcclInTooltips(true), bShowLine(true),
 bXPStyle(true), bIsPopup(false), bIsMenuOpen(false),
 openMenuIndex(-1),
 imageSize(DEFAULT_BITMAP_WIDTH, DEFAULT_BITMAP_HEIGHT)
{
  module = pModule ? pModule : ::Module;

  XPSelectColor = HLS_TRANSFORM(TColor::SysHighlight, 70, -57);

  imageSize.cx -= 1;
  imageSize.cy -= 1;

  imageList = new TImageList(imageSize, ILC_COLOR4 | ILC_MASK, 0, 1);
  imageList->SetBkColor(CLR_NONE);
}

TBmpMenu::~TBmpMenu()
{
  imageList->RemoveAll();

  delete imageList;
}

void
TBmpMenu::EvInitMenuPopup(HMENU hPopupMenu, uint index, bool sysMenu)
{
  if (!sysMenu) {
    TMenuItemInfo mii(MIIM_DATA);
    ::GetMenuItemInfo(hPopupMenu, 0, TRUE, &mii);

    TBmpMenuItem* pItem = (TBmpMenuItem*)mii.dwItemData;

    if (pItem && pItem->GetCommandId() != -4) {
      bIsMenuOpen = true;

      openMenuIndex = index;

      // Redraw the menu bar
      //
      TRect rect;
      ::GetMenuItemRect(*parent, GetParentMenu(hPopupMenu), index, &rect);

      TPoint point(0, 0);
      parent->ClientToScreen(point);

      rect.Offset(-point.x, -point.y);

      ::RedrawWindow(*parent, &rect, 0, RDW_FRAME | RDW_INVALIDATE);
    }
  }
}

void TBmpMenu::EvUnInitMenuPopup(HMENU hPopupMenu)
{
  HMENU hMenu = GetParentMenu(hPopupMenu);

  if (hMenu) {
    TMenuItemInfo mii(MIIM_DATA);
    ::GetMenuItemInfo(hMenu, openMenuIndex, TRUE, &mii);

    TBmpMenuItem* pItem = (TBmpMenuItem*)mii.dwItemData;

    if (pItem && pItem->GetCommandId() == -4) {
      bIsMenuOpen = false;

      // Redraw the menu bar
      //
      TRect rect;
      ::GetMenuItemRect(*parent, hMenu, openMenuIndex, &rect);

      TPoint point(0, 0);
      parent->ClientToScreen(point);

      rect.Offset(-point.x, -point.y);

      ::RedrawWindow(*parent, &rect, 0, RDW_FRAME | RDW_INVALIDATE);

      openMenuIndex = -1;
    }
  }
}

void
TBmpMenu::RemapMenu(HMENU hMenu, bool isPopup)
{
  SynchronizeMenuItems();

  if (::IsMenu(hMenu)) {
    bIsPopup = isPopup;

    RemapMenuRecursive(hMenu);
  }
}

bool
TBmpMenu::SetMenuItemText(HMENU hMenu, uint pos, const char* text)
{
  char buffer[MAX_STRING_LENGTH];
  strcpy(buffer, (text ? text : ""));

  TMenuItemInfo mii(MIIM_DATA | MIIM_ID | MIIM_STATE | MIIM_SUBMENU | MIIM_TYPE);
  if (::GetMenuItemInfo(hMenu, pos, TRUE, &mii) &&
      (mii.fType & MFT_OWNERDRAW) && mii.dwItemData) {
    TIListIteratorImp<TBmpMenuItem> iterator(menuItems);

    while (iterator) {
      TBmpMenuItem* pItem = iterator++;

      if (pItem->GetHandle() == hMenu && pItem->GetPosition() == pos) {
        if (pItem->GetCommandId() > 0) {
          if ((uint)pItem->GetCommandId() == mii.wID) {
            if (!pItem->CompareText(buffer))
              return true;  // Nothing to change

            pItem->SetText(buffer);

            return ::ModifyMenu(hMenu, pos, MF_BYPOSITION | MF_OWNERDRAW |
                                mii.fState, mii.wID, (LPCTSTR)pItem);
          }
        }
        else if (mii.hSubMenu) {
          if (!pItem->CompareText(buffer))
            return true;  // Nothing to change

          pItem->SetText(buffer);

          return ::ModifyMenu(hMenu, pos, MF_BYPOSITION | MF_OWNERDRAW |
                              mii.fState, -1, (LPCTSTR)pItem);
        }
      }
    }
  }

  return false;
}

bool
TBmpMenu::DrawItem(DRAWITEMSTRUCT& dis)
{
  char buffer[MAX_STRING_LENGTH];

  bool res = false;

  if (dis.CtlType == ODT_MENU) {
    UINT state = dis.itemState;

    bool bEnabled  = !(state & ODS_DISABLED);
    bool bChecked  =  (state & ODS_CHECKED);
    bool bHotLight =  (state & ODS_HOTLIGHT);
    bool bNoAccel  =  (state & ODS_NOACCEL);
    bool bSelected =  (state & ODS_SELECTED);

    bool bDefault    = false;
    bool bRadioCheck = false;

    TBmpMenuItem* pItem = (TBmpMenuItem*)dis.itemData;

    if (pItem) {
      // If it is not a submenu item,
      // get the menu item info
      //
      TMenuItemInfo mii(MIIM_STATE | MIIM_TYPE);

      if (pItem->GetCommandId() >= 0) {
        bool bItemInfoValid = ::GetMenuItemInfo((HMENU)dis.hwndItem, dis.itemID,
                                                FALSE, &mii);
        if (bItemInfoValid) {
          if (mii.fState & MFS_DEFAULT)
            bDefault = true;

          if (mii.fType & MFT_RADIOCHECK)
            bRadioCheck = true;
        }
      }

      TDC dc(dis.hDC);

      // Get the actual system wide menu font
      //
      TFont* pFont;

      NONCLIENTMETRICS nm;
      nm.cbSize = sizeof(NONCLIENTMETRICS);

      if (::SystemParametersInfo(SPI_GETNONCLIENTMETRICS, nm.cbSize, &nm, 0)) {
        if (bDefault)
          nm.lfMenuFont.lfWeight = FW_BOLD;

        pFont = new TFont(&nm.lfMenuFont);
      }
      else
        pFont = new TFont((HFONT)::GetStockObject(DEFAULT_GUI_FONT));

      dc.SelectObject(*pFont);

      TColor nrBmpBarColor;  // Normal    bitmap bar color
      TColor ltBmpBarColor;  // Lightened bitmap bar color
      TColor dkBmpBarColor;  // Darkened  bitmap bar color

      TColor textBarColor;
      TColor textColor;

      if (bXPStyle) {
        if (bEnabled && bSelected) {
          nrBmpBarColor = XPSelectColor;

          textBarColor = XPSelectColor;
        }
        else {
          nrBmpBarColor = TColor::Sys3dFace;
          ltBmpBarColor = LightenColor(nrBmpBarColor, 20);
          dkBmpBarColor = DarkenColor (nrBmpBarColor, 40);

          textBarColor = TColor::Sys3dHilight;
        }

        textColor = bEnabled ? TColor::SysMenuText : TColor::SysGrayText;
      }
      else {
        nrBmpBarColor = (bSelected && pItem->GetImageIndex() < 0) ?
                        TColor::SysHighlight : TColor::Sys3dFace;
        textBarColor  = bSelected ? TColor::SysHighlight : TColor::Sys3dFace;

        if (bEnabled)
          textColor = bSelected ? TColor::SysHighlightText :
                                  TColor::SysMenuText;
        else
          textColor = TColor::SysGrayText;
      }

      TRect rc(dis.rcItem);

      TRect bmpBarRect(rc);

      bmpBarRect.left  = 0;
      bmpBarRect.right = DEFAULT_BITMAP_WIDTH + 2 * BITMAP_EXTRA_WIDTH + 1;

      TRect imageRect(rc);

      imageRect.left   = BITMAP_EXTRA_WIDTH + 1;
      imageRect.top   += (rc.Height()   - DEFAULT_BITMAP_HEIGHT) / 2;
      imageRect.right  = imageRect.left + DEFAULT_BITMAP_WIDTH   - 1;
      imageRect.bottom = imageRect.top  + DEFAULT_BITMAP_HEIGHT  - 1;

      TRect textBarRect(rc);

      textBarRect.left = bmpBarRect.right;

      if (pItem->GetCommandId() == -3)  {  // Is a separator
        if (bXPStyle)
          DrawGradientRect(dc, bmpBarRect, ltBmpBarColor, dkBmpBarColor,
                           GRADIENT_FILL_RECT_H);
        else
          FillSolidRect(dc, bmpBarRect, nrBmpBarColor);

        FillSolidRect(dc, textBarRect, textBarColor);

        dc.SelectObject(TPen(TColor::Sys3dShadow));

        dc.MoveTo((bXPStyle ? textBarRect.left : bmpBarRect.left) + 2,
                  bmpBarRect.top + 2);
        dc.LineTo(textBarRect.right - 2, bmpBarRect.top + 2);

        dc.RestorePen();

        dc.SelectObject(TPen(TColor::Sys3dHilight));

        dc.MoveTo((bXPStyle ? textBarRect.left : bmpBarRect.left) + 2,
                  bmpBarRect.top + 3);
        dc.LineTo(textBarRect.right - 2, bmpBarRect.top + 3);

        dc.RestorePen();
      }
      else if (pItem->GetCommandId() == -4) {  // Is a title item
        pItem->GetText(buffer, sizeof(buffer), bNoAccel);

        dc.SetBkMode(TRANSPARENT);

        if (bXPStyle) {
          FillSolidRect(dc, rc, TColor::Sys3dFace);  // Erase title bar

          if (!bSelected || !bIsMenuOpen)
            rc.right -= 4;
             
          if (bEnabled) {
            if (bSelected && bIsMenuOpen) {
              TColor barColor(dc.GetPixel(rc.right - 1, rc.top));

              int h = rc.Height();

              for (int x = 1; x <= 4; x++) {
                int right = rc.right - x;

                for (int y = 0; y < 4; y++) {
                  if (barColor == CLR_INVALID)
                    barColor = dc.GetPixel(right, rc.top + y);

                  dc.SetPixel(right, rc.top + y, barColor);
                }

                for (int y = 4; y < 8; y++)
                  dc.SetPixel(right, rc.top + y,
                              DarkenColor(barColor, (BYTE)(6 * x * (y - 3))));

                for (int y = 8; y < h ; y++)
                  dc.SetPixel(right, rc.top + y,
                              DarkenColor(barColor, (BYTE)(30 * x)));
              }

              rc.right -= 4;

              FillSolidRect(dc, rc, TColor::SysWindow);

              dc.SelectObject(TPen(DarkenColor(TColor::Sys3dFace, 64),
                                   0, PS_SOLID));

              dc.MoveTo(rc.left,  rc.bottom);

              dc.LineTo(rc.left,  rc.top);
              dc.LineTo(rc.right, rc.top);
              dc.LineTo(rc.right, rc.bottom);
              
              dc.RestorePen();
            }
            else if ((bSelected && !bIsMenuOpen) || bHotLight) {
              FillSolidRect(dc, rc, XPSelectColor);
              Draw3dRect(dc, rc, TColor::SysActiveCaption,
                         TColor::SysActiveCaption);
            }
            else
              FillSolidRect(dc, rc, TColor::Sys3dFace);

            dc.SetTextColor(TColor::SysMenuText);
          }
          else {
            FillSolidRect(dc, rc, TColor::Sys3dFace);

            dc.SetTextColor(TColor::SysGrayText);
          }
        }
        else {
          if (bEnabled) {
            if (bSelected || bHotLight) {
              FillSolidRect(dc, rc, TColor::SysHighlight);

              dc.SetTextColor(TColor::SysHighlightText);
            }
            else {
              FillSolidRect(dc, rc, TColor::Sys3dFace);

              dc.SetTextColor(TColor::SysMenuText);
            }
          }
          else {
            FillSolidRect(dc, rc, TColor::Sys3dFace);

            dc.SetTextColor(TColor::SysGrayText);
          }
        }

        dc.DrawText(buffer, -1, rc, DT_CENTER | DT_SINGLELINE | DT_VCENTER);
      }
      else {
        // Draw menu item
        //
        int oldBkMode = dc.SetBkMode(TRANSPARENT);

        if (bXPStyle && (!bEnabled || !bSelected))
          DrawGradientRect(dc, bmpBarRect, ltBmpBarColor, dkBmpBarColor,
                           GRADIENT_FILL_RECT_H);
        else
          FillSolidRect(dc, bmpBarRect, nrBmpBarColor);

        FillSolidRect(dc, textBarRect, textBarColor);

        if (bSelected) {
          if (bEnabled && bXPStyle)
            Draw3dRect(dc, rc, TColor::SysActiveCaption,
                       TColor::SysActiveCaption);
        }
        else if (pItem->GetImageIndex() >= 0 && bChecked) {
          // Draw 50% mask
          //
          const uint8 mask[8] = {
            0xAA, 0x55,
            0xAA, 0x55,
            0xAA, 0x55,
            0xAA, 0x55
          };

          THatch8x8Brush brush(mask, TColor::Sys3dHilight, TColor::Sys3dFace);
          dc.FillRect(bmpBarRect.InflatedBy(-1, -1), brush);
        }

        if (pItem->GetImageIndex() >= 0) {
          if (bChecked)
            Draw3dRect(dc, bmpBarRect.InflatedBy(-1, 0), TColor::Sys3dShadow,
                       TColor::Sys3dHilight);
          else if (bEnabled && bSelected && !bXPStyle)
            Draw3dRect(dc, bmpBarRect.InflatedBy(-1, 0), TColor::Sys3dHilight,
                       TColor::Sys3dShadow);

          if (bEnabled)
            imageList->Draw(pItem->GetImageIndex(), dc,
                            imageRect.left, imageRect.top,
                            bChecked ? ILD_TRANSPARENT : ILD_NORMAL);
          else {
            HICON hIcon = imageList->GetIcon(pItem->GetImageIndex(),
                                             ILD_NORMAL);
            ::DrawState((HDC)dc, NULL, NULL, (LPARAM)hIcon, 0,
                        imageRect.left, imageRect.top,
                        imageRect.Width(), imageRect.Height(),
                        DST_ICON | DSS_DISABLED);
            ::DestroyIcon(hIcon);
          }
        }
        else if (bChecked) {
          if (bDrawRecessed)
            Draw3dRect(dc, bmpBarRect.InflatedBy(-1, -1), TColor::Sys3dShadow,
                       TColor::Sys3dHilight);

          if (bRadioCheck)
            DrawRadioDot(dc, imageRect.left + 6, imageRect.top + 5,
                         TColor::SysMenuText);
          else
            DrawCheckMark(dc, imageRect.left + 6, imageRect.top + 5,
                          TColor::SysMenuText);
        }

        if (bEnabled && bSelected && bXPStyle)
          Draw3dRect(dc, rc, TColor::SysActiveCaption,
                     TColor::SysActiveCaption);

        pItem->GetText(buffer, sizeof(buffer), bNoAccel);

        TSize sz;
        dc.GetTextExtent(buffer, strlen(buffer), sz);

        int ay = (textBarRect.Height() - sz.cy) / 2;

        TRect textRect(textBarRect);
        textRect.left  += XP_EXTRA_WIDTH + 2;
        textRect.top   += ay;
        textRect.right -= 15;

        char* ptr;

        ptr = strchr(buffer, '\t');

        if (!ptr)
          ptr = strchr(buffer, '\b');

        if (ptr) {
          *ptr = 0;

          if (!bXPStyle && !bEnabled && !bSelected) {
            // Draw a shaded text behind the actual text
            //
            TRect shadedRect(textRect.InflatedBy(-1, -1));

            dc.SetTextColor(TColor::Sys3dHilight);
            dc.DrawText(buffer,  -1, shadedRect, DT_LEFT  | DT_VCENTER);
            dc.DrawText(ptr + 1, -1, shadedRect, DT_RIGHT | DT_VCENTER);
          }

          dc.SetTextColor(textColor);
          dc.DrawText(buffer,  -1, textRect, DT_LEFT  | DT_VCENTER);
          dc.DrawText(ptr + 1, -1, textRect, DT_RIGHT | DT_VCENTER);
        }
        else {
          if (!bXPStyle && !bEnabled && !bSelected) {
            // Draw a shaded text behind the actual text
            //
            dc.SetTextColor(TColor::Sys3dHilight);
            dc.DrawText(buffer, -1, textRect.InflatedBy(-1, -1),
                        DT_EXPANDTABS | DT_LEFT | DT_VCENTER);
          }

          dc.SetTextColor(textColor);
          dc.DrawText(buffer, -1, textRect,
                      DT_EXPANDTABS | DT_LEFT | DT_VCENTER);
        }

        dc.SetBkMode(oldBkMode);
      }

      if (bShowLine && pItem->GetCommandId() != -4) {
        dc.SelectObject(TPen(TColor::SysActiveCaption));

        dc.MoveTo(textBarRect.TopLeft   ());
        dc.LineTo(textBarRect.BottomLeft());

        dc.RestorePen();
      }

      dc.RestoreFont();

      delete pFont;
    }

    res = true;
  }

  return res;
}

bool
TBmpMenu::MeasureItem(MEASUREITEMSTRUCT& mis)
{
  char buffer[MAX_STRING_LENGTH];

  bool res = false;

  if (mis.CtlType == ODT_MENU) {
    TBmpMenuItem* pItem = (TBmpMenuItem*)mis.itemData;

    if (pItem) {
      if (pItem->GetCommandId() == -3) {  // Is a separator
        mis.itemHeight = 6;
        mis.itemWidth  = 10;
      }
      else if (pItem->GetText(buffer, sizeof(buffer)) != 0) {
        TClientDC dc(0);

        // Get the actual system wide menu font
        //
        TFont* pFont;

        NONCLIENTMETRICS nm;
        nm.cbSize = sizeof(NONCLIENTMETRICS);

        if (::SystemParametersInfo(SPI_GETNONCLIENTMETRICS, nm.cbSize, &nm, 0))
          pFont = new TFont(&nm.lfMenuFont);
        else
          pFont = new TFont((HFONT)::GetStockObject(DEFAULT_GUI_FONT));

        dc.SelectObject(*pFont);

        if (pItem->GetCommandId() == -4) {  // Is a title item
          TRect rect(0, 0, 0, 0);
          dc.DrawText(buffer, -1, rect,
                      DT_CALCRECT | DT_CENTER | DT_SINGLELINE | DT_VCENTER);

          mis.itemHeight = rect.Height();
          mis.itemWidth  = rect.Width ();

          if (bXPStyle)
            mis.itemWidth += 4;
        }
        else {
          int h = ::GetSystemMetrics(SM_CYMENU);
          mis.itemHeight = (h > imageSize.cy + 3) ? h : imageSize.cy + 3;

          TSize sz;
          dc.GetTabbedTextExtent(buffer, strlen(buffer), 0, 0, sz);

          mis.itemWidth = mis.itemHeight + sz.cx + 3 * sz.cx / strlen(buffer) +
                          BITMAP_EXTRA_WIDTH + 2;

          if (bXPStyle)
            mis.itemWidth += XP_EXTRA_WIDTH * 2;
        }

        dc.RestoreFont();

        delete pFont;
      }
      else {
        mis.itemHeight = imageSize.cy + 3;
        mis.itemWidth  = 100;
      }
    }

    res = true;
  }

  return res;
}

bool
TBmpMenu::FindKeyboardShortcut(uint nChar, uint /* nFlags */, HMENU hMenu,
                               LRESULT& lRes)
{
  int ch = tolower(nChar);

  int n = ::GetMenuItemCount(hMenu);

  for (int i = 0; i < n; i++) {
    TMenuItemInfo mii(MIIM_DATA | MIIM_TYPE);

    if (::GetMenuItemInfo(hMenu, i, TRUE, &mii) &&
        (mii.fType & MFT_OWNERDRAW)) {
      TBmpMenuItem* pItem = (TBmpMenuItem*)mii.dwItemData;

      if (pItem) {
        char buffer[MAX_STRING_LENGTH];
        pItem->GetText(buffer, sizeof(buffer));

        char* ptr = strchr(buffer, '&');

        if (ptr && tolower(*(ptr + 1)) == ch) {
          lRes = MAKELONG(i, 2);

          return true;
        }
      }
    }
  }

  return false;
}

int
TBmpMenu::PreGetHintText(HMENU hMenu, uint id, char* buf, int size,
                         THintText hintType)
{
  int numBytes = 0;

  if (hMenu && IsMenu(hMenu) && hintType == htTooltip) {
    // For tooltips, we first look if the same id exists on
    // the bitmap menu and use the associated menu string
    //
    bool found     = false;
    bool acclValid = true;

    TIListIteratorImp<TBmpMenuItem> iterator(menuItems);

    while (iterator) {
      TBmpMenuItem* pItem = iterator++;

      if (pItem->GetCommandId() == (int)id) {
        if (pItem->GetText(buf, size, true) == size) {
          buf[size - 1] = 0;

          acclValid = false;
        }

        found = true;

        break;
      }
    }

    if (found) {
      char buffer[MAX_STRING_LENGTH];

      char* ptr;

      // Trim accelerator text such as "\tCtrl+X" if any
      // Accelerator can be \b instead of \t
      //
      buffer[0] = 0;

      ptr = strchr(buf, '\t');

      if (!ptr)
        ptr = strchr(buf, '\b');

      if (ptr) {
        if (bShowAcclInTooltips && acclValid)
          // Save the accelerator string to show in tooltips
          //
          strcpy(buffer, ptr + 1);

        *ptr = 0;
      }

      // Trim trailing dots, e.g. "..."
      //
      for (ptr = buf + strlen(buf) - 1; ptr >= buf; ptr--)
        if (*ptr == '.')
          *ptr = 0;
        else
          break;

      // Show the accelerator in tootip
      //
      if (bShowAcclInTooltips && acclValid &&
          buffer[0] && (strlen(buf) + strlen(buffer) + 3 < (uint)size)) {
        strcat(buf, " (");
        strcat(buf, buffer);
        strcat(buf, ")");
      }

      numBytes = strlen(buf);
    }
  }

  return numBytes;
}

int
TBmpMenu::FindImage(int cmd) const
{
  TIListIteratorImp<TBmpMenuImage> iterator(menuImages);

  while (iterator) {
    TBmpMenuImage* image = iterator++;

    if (image->GetCommandId() == cmd)
      return image->GetImageIndex();
  }

  return -1;
}

HMENU
TBmpMenu::GetParentMenu(HMENU child) const
{
  TIListIteratorImp<TBmpMenuItem> iterator(menuItems);

  while (iterator) {
    HMENU hMenu = (iterator++)->GetHandle();

    int n = ::GetMenuItemCount(hMenu);

    for (int i = 0; i < n; i++)
      if (::GetSubMenu(hMenu, i) == child)
        return hMenu;
  }

  return 0;
}

void
TBmpMenu::RemapMenuRecursive(HMENU hMenu)
{
  char buffer[MAX_STRING_LENGTH];

  static int recurse = 0;

  recurse++;

  int n = ::GetMenuItemCount(hMenu) - 1;

  for (int i = n; i >= 0; i--) {
    uint itemId = ::GetMenuItemID(hMenu, i);

    if (itemId == (uint)-1) {
      HMENU popup = ::GetSubMenu(hMenu, i);

      if (popup)
        RemapMenuRecursive(popup);

      if (bIsPopup || recurse > 0) {
        if (::GetMenuString(hMenu, i, buffer, sizeof(buffer), MF_BYPOSITION)) {
          TBmpMenuItem* pItem;
          pItem = AddMenuItem(hMenu, buffer,
                              (!bIsPopup && recurse == 1) ? -4 : -2, i);
          ::ModifyMenu(hMenu, i, MF_BYPOSITION | MF_OWNERDRAW, (uint)-1,
                       (LPCSTR)pItem);
        }
      }
    }
    else {
      TBmpMenuItem* pItem;

      uint state = ::GetMenuState(hMenu, i, MF_BYPOSITION);

      if (!(state & MF_BITMAP)) {
        if (!(state & MF_OWNERDRAW)) {
          if ((state & MF_SEPARATOR) || itemId == 0) {
            pItem = AddMenuItem(hMenu, "--", -3, i);
            ::ModifyMenu(hMenu, i, state | MF_BYPOSITION | MF_OWNERDRAW,
                         (LPARAM)itemId, (LPCSTR)pItem);
          }
          else if (::GetMenuString(hMenu, i, buffer, sizeof(buffer),
                                   MF_BYPOSITION)) {
            LoadBitmapFromResource(itemId);
            pItem = AddMenuItem(hMenu, buffer, itemId, i);
            ::ModifyMenu(hMenu, i, state | MF_BYPOSITION | MF_OWNERDRAW,
                         (LPARAM)itemId, (LPCSTR)pItem);
          }
        }
        else {
          // We have to make sure that the position
          // of owner drawn menu item is correct
          //
          TMenuItemInfo mii(MIIM_DATA);
          ::GetMenuItemInfo(hMenu, i, TRUE, &mii);

          TBmpMenuItem* pItem = (TBmpMenuItem*)mii.dwItemData;

          if (pItem && pItem->GetPosition() != (uint)i)
            pItem->SetPosition(i);
        }
      }
    }
  }

  recurse--;
}

void
TBmpMenu::DrawCheckMark(TDC& dc, int x, int y, TColor color)
{
  // ......X
  // .....XX
  // X...XXX
  // XX.XXX.
  // XXXXX..
  // .XXX...
  // ..X....

  dc.SetPixel(x, y + 2, color);
  dc.SetPixel(x, y + 3, color);
  dc.SetPixel(x, y + 4, color);

  x++;

  dc.SetPixel(x, y + 3, color);
  dc.SetPixel(x, y + 4, color);
  dc.SetPixel(x, y + 5, color);

  x++;

  dc.SetPixel(x, y + 4, color);
  dc.SetPixel(x, y + 5, color);
  dc.SetPixel(x, y + 6, color);

  x++;

  dc.SetPixel(x, y + 3, color);
  dc.SetPixel(x, y + 4, color);
  dc.SetPixel(x, y + 5, color);

  x++;

  dc.SetPixel(x, y + 2, color);
  dc.SetPixel(x, y + 3, color);
  dc.SetPixel(x, y + 4, color);

  x++;

  dc.SetPixel(x, y + 1, color);
  dc.SetPixel(x, y + 2, color);
  dc.SetPixel(x, y + 3, color);

  x++;

  dc.SetPixel(x, y,     color);
  dc.SetPixel(x, y + 1, color);
  dc.SetPixel(x, y + 2, color);
}

void
TBmpMenu::DrawRadioDot(TDC& dc, int x, int y, TColor color)
{
  dc.SelectObject(TBrush(color));

  dc.SelectObject(TPen(color, 0));

  dc.Ellipse(x, y, x + 7, y + 7);

  dc.RestorePen();

  dc.RestoreBrush();
}

TBmpMenuItem*
TBmpMenu::AddMenuItem(HMENU hMenu, const char* text, int cmd, uint pos)
{
  char buffer[MAX_STRING_LENGTH];
  strcpy(buffer, (text ? text : ""));

  TBmpMenuItem* pNewItem;
  pNewItem = new TBmpMenuItem(hMenu, (cmd > 0 ? FindImage(cmd) : -1), cmd, pos,
                              buffer);

  TIListIteratorImp<TBmpMenuItem> iterator(menuItems);

  while (iterator) {
    TBmpMenuItem* pItem = iterator++;

    if (*pItem == *pNewItem) {
      delete pNewItem;

      return pItem;
    }
  }

  menuItems.Add(pNewItem);

  return pNewItem;
}

void
TBmpMenu::AddMenuImage(int idx, int cmd)
{
  if (idx < 0)
    return;

  TBmpMenuImage* pImage = new TBmpMenuImage(idx, cmd);

  menuImages.Add(pImage);
}

bool
TBmpMenu::LoadBitmapFromResource(unsigned int resId)
{
  // Check if the bitmap is excluded
  //
  if (excludedBitmaps.Find(resId))
    return false;

  // Add only one bitmap with resId to the image list
  //
  if (FindImage(resId) < 0) {
    // Load bitmap from module and add it to the list
    //
    TBitmap bitmap(module->LoadBitmap(resId));

    if ((HBITMAP)bitmap) {
      int index = imageList->Add(bitmap, RGB(192, 192, 192));

      AddMenuImage(index, resId);

      return true;
    }
  }

  return false;
}

void
TBmpMenu::SynchronizeMenuItems()
{
  TIListIteratorImp<TBmpMenuItem> iterator(menuItems);

  while (iterator) {
    TBmpMenuItem* pItem = iterator++;

    if (!::IsMenu(pItem->GetHandle()))
      menuItems.Detach(pItem, TShouldDelete::Delete);
  }

  iterator.Restart();

  while (iterator) {
    bool bResult;

    TBmpMenuItem* pItem = iterator++;

    TMenuItemInfo mii(MIIM_DATA | MIIM_ID | MIIM_STATE | MIIM_SUBMENU |
                      MIIM_TYPE);

    if (pItem->GetCommandId() > 0)
      bResult = ::GetMenuItemInfo(pItem->GetHandle(), pItem->GetCommandId(),
                                  false, &mii);
    else
      bResult = ::GetMenuItemInfo(pItem->GetHandle(), pItem->GetPosition(),
                                  true, &mii);

    if (!bResult)
      menuItems.Detach(pItem, TShouldDelete::Delete);
    else {
      bool bDelete = false;

      if (pItem->GetCommandId() > 0) {  // Intern item is a simple item
        if (mii.hSubMenu) {
          // If windows item is a submenu,
          // delete intern item
          //
          bDelete = true;
        }
        else if ((UINT)pItem->GetCommandId() != mii.wID) {
          // If windows command changes,
          // delete intern item
          //
          bDelete = true;
        }
      }
      else if (pItem->GetCommandId() != -3 && mii.hSubMenu == 0) {
        // If intern item is a submenu and windows item
        // is a simple item, delete intern item
        //
        bDelete = true;
      }

      if (bDelete == false && !(mii.fType & MFT_OWNERDRAW)) {
        // Windows item is not ownerdrawn
        //
        if ((mii.fType & MFT_STRING) && mii.dwTypeData) {
          // If windows item is a string item, try to
          // synchronize intern and windows items
          //
          pItem->SetText(mii.dwTypeData);

          UINT cmd;
          cmd = pItem->GetCommandId() > 0 ? pItem->GetCommandId() : (UINT)-1;

          ::ModifyMenu(pItem->GetHandle(), pItem->GetPosition(), mii.fState |
                       MF_BYPOSITION | MF_OWNERDRAW, cmd, (LPCTSTR)pItem);
        }
      }

      if ((mii.fType & MFT_OWNERDRAW) && !mii.dwItemData)
        // If windows item is ownerdrawn and has
        // no valid data, delete intern item
        //
        bDelete = true;

      if (bDelete)
        menuItems.Detach(pItem, TShouldDelete::Delete);
    }
  }
}

void
TBmpMenu::DrawGradientRect(TDC& dc, TRect& rect, TColor color1,
                              TColor color2, ULONG mode)
{
  TRIVERTEX vertices[2];

  vertices[0].x     = rect.left;
  vertices[0].y     = rect.top;
  vertices[0].Red   = (COLOR16)(color1.Red  () << 8);
  vertices[0].Green = (COLOR16)(color1.Green() << 8);
  vertices[0].Blue  = (COLOR16)(color1.Blue () << 8);
  vertices[0].Alpha = 0x0000;

  vertices[1].x     = rect.right;
  vertices[1].y     = rect.bottom;
  vertices[1].Red   = (COLOR16)(color2.Red  () << 8);
  vertices[1].Green = (COLOR16)(color2.Green() << 8);
  vertices[1].Blue  = (COLOR16)(color2.Blue () << 8);
  vertices[1].Alpha = 0x0000;

  GRADIENT_RECT gRect;
  gRect.UpperLeft  = 0;
  gRect.LowerRight = 1;

  MSImage32.GradientFill((HDC)dc, vertices, 2, &gRect, 1, mode);
}

