// ****************************************************************************
// Copyright (C) 1998 by Dieter Windau
// All rights reserved
//
// BmpMenu.cpp: implementation file
// Version:     1.2
// Date:        07.08.1998
// Author:      Dieter Windau
//
// TBmpMenu is a freeware OWL class that shows bitmaps
// on the left side of menus like MS Office 97
//
// Portions of code are based on MFC class CMenuSpawn written by Iuri
// Apollonio and BCMenu written by Brent Corkum. Very special thanks.
//
// You are free to use/modify this code but leave this header intact.
// May not be sold for profit.
//
// Tested with Borland C++ 5.02, OWL 5.02 under Windows NT 4.0 SP3 but I think
// the class would work with Windows 95 too.
// This file is provided "as is" with no expressed or implied warranty.
// Use at your own risk.
//
// Please send me bug reports, bug fixes, enhancements, requests, etc., and
// I'll try to keep this in next versions:
//   EMail: dieter.windau@usa.net
//   Web:   http://www.members.aol.com/SoftEngage
// ****************************************************************************

// 28/02/2002  Luigi Bianchi made some changes to make this code
// work without BIDS and started to add Office XP Look and Feel
// CHeck updates at www.luigibianchi.com/owl
// 02/03/2002	Luigi Bianchi, change the way in which XPExtraWidth
// was computed due to 1 pixel error when using 20 * 20 bitmaps

// ****************************************************************************
// 4/05/2009 Mohsen Jahanshahi added still more feel and look of windows XP
// With appology to Luigi Bianchi for using BIDS in this code
//

#if !defined(TOOL_BMPMENU_H)
#define TOOL_BMPMENU_H

#if !defined(OWL_COLOR_H)
# include <owl/color.h>
#endif
#if !defined(OWL_DC_H)
# include <owl/dc.h>
#endif
#if !defined(OWL_DECFRAME_H)
# include <owl/decframe.h>
#endif
#if !defined(OWL_IMAGELST_H)
# include <owl/imagelst.h>
#endif
#if !defined(OWL_MODULE_H)
# include <owl/module.h>
#endif
#if !defined(CLASSLIB_LISTIMP_H)
# include <classlib/listimp.h>
#endif
#if !defined(TOOL_MSIMG32_H)
# include "msimg32.h"
#endif

#if !defined(WM_UNINITMENUPOPUP)
# define WM_UNINITMENUPOPUP 0x0125
#endif

#if !defined(ODS_HOTLIGHT)
# define ODS_HOTLIGHT 0x0040
#endif
#if !defined(ODS_NOACCEL)
# define ODS_NOACCEL  0x0100
#endif

//
// class TBmpMenuItem
// ~~~~~ ~~~~~~~~~~~~
// This class encapsulates common information
// about menu items
//
class TBmpMenuItem {
  public:
    // Constructors
    //
    TBmpMenuItem();
    TBmpMenuItem(HMENU handle, int idx, int idc, uint pos, const char* txt);

    // Destructor
    //
   ~TBmpMenuItem();

    // Accessors
    //
    int CompareText(const char* buffer) const;

    HMENU GetHandle() const;

    int GetImageIndex() const;
    int GetCommandId () const;

    uint GetPosition() const;

    int GetText(char* buffer, int size, bool noAccel = false) const;

    bool operator ==(const TBmpMenuItem& other) const;

    // Mutators
    //
    void SetHandle(HMENU handle);

    void SetImageIndex(int idx);
    void SetCommandId (int idc);

    void SetPosition(uint pos);

    void SetText(const char* txt);

  private:
    HMENU hMenu;     // The handle of menu to which this item belongs

    int imageIndex;
    int commandId;

    uint position;

    char* text;
};

//
// class TBmpMenuImage
// ~~~~~ ~~~~~~~~~~~~~
// This class encapsulates information about
// bitmaps associated to menu items
//
class TBmpMenuImage {
  public:
    // Constructors
    //
    TBmpMenuImage();
    TBmpMenuImage(int idx, int idc);

    // Accessors
    //
    int GetImageIndex() const;
    int GetCommandId () const;

    bool operator ==(const TBmpMenuImage& other) const;

    // Mutators
    //
    void SetImageIndex(int idx);
    void SetCommandId (int idc);

  private:
    int imageIndex;
    int commandId;
};

//
// class TBmpMenu
// ~~~~~ ~~~~~~~~
// This class manages the operations needed
// to prepare and show bitmap menus
//
class TBmpMenu {
  public:
    // Constructor & destructor
    //
    TBmpMenu(TWindow* pWindow, TModule* pModule = 0);
   ~TBmpMenu();

    // Accessors
    //
    bool DrawRecessed() const;

    bool ShowAcclInTooltips() const;

    bool ShowLine() const;

    bool XPStyle() const;

    // Mutators
    //
    void DrawRecessed(bool flag);

    void ShowAcclInTooltips(bool flag);

    void ShowLine(bool flag);

    void XPStyle(bool flag);

    void ExcludeBitmap(unsigned int resId);
    void IncludeBitmap(unsigned int resId);

    void EvInitMenuPopup  (HMENU hPopupMenu, uint index, bool sysMenu);
    void EvUnInitMenuPopup(HMENU hPopupMenu);

    void RemapMenu(HMENU hMenu, bool isPopup = false);

    bool SetMenuItemText(HMENU hMenu, uint pos, const char* text);

    bool DrawItem   (DRAWITEMSTRUCT&    dis);
    bool MeasureItem(MEASUREITEMSTRUCT& mis);

    bool FindKeyboardShortcut(uint nChar, uint nFlags, HMENU hMenu,
                              LRESULT& lRes);

    int PreGetHintText(HMENU hMenu, uint id, char* buf, int size,
                       THintText hintType);

  private:
    TWindow* parent;           // Parent window

    TModule* module;

    TMSImage32Module& MSImage32;

    bool bDrawRecessed;        // Show top level menu items recessed
    bool bShowAcclInTooltips;  // Show accelerators in tooltips
    bool bShowLine;            // Show vertical line
    bool bXPStyle;             // Show menus using XP styles
    bool bIsPopup;             // Is the remap menu a popup menu
    bool bIsMenuOpen;          // Is any of drop down menus open

    int openMenuIndex;         // The index of open drop down menu

    TColor XPSelectColor;

    TSize imageSize;           // The size that is used for all bitmaps

    TListImp<unsigned int>   excludedBitmaps;

    TIListImp<TBmpMenuItem>  menuItems;
    TIListImp<TBmpMenuImage> menuImages;

    TImageList* imageList;     // Contains all used bitmaps

    // Accessors
    //
    int FindImage(int cmd) const;

    HMENU GetParentMenu(HMENU child) const;

    // Mutators
    //
    void RemapMenuRecursive(HMENU hMenu);

    void DrawCheckMark(TDC& dc, int x, int y, TColor color);

    void DrawRadioDot(TDC& dc, int x, int y, TColor color);

    TBmpMenuItem* AddMenuItem(HMENU hMenu, const char* text, int cmd, uint pos);

    void AddMenuImage(int idx, int cmd);

    bool LoadBitmapFromResource(unsigned int resId);

    void SynchronizeMenuItems();

    void DrawGradientRect(TDC& dc, TRect& rect, TColor color1, TColor color2,
                          ULONG mode);
};

// TBmpMenu inline implementation
//
inline
TBmpMenuItem::TBmpMenuItem()
:
 hMenu(0),
 imageIndex(0), commandId(0),
 position(0),
 text(0)
{
}

inline
TBmpMenuItem::TBmpMenuItem(HMENU handle, int idx, int idc, uint pos,
                           const char* txt)
:
 hMenu(handle),
 imageIndex(idx), commandId(idc),
 position(pos)
{
  text = new char[strlen(txt) + 1];
  strcpy(text, txt);
}

inline
TBmpMenuItem::~TBmpMenuItem()
{
  if (text)
    delete[] text;
}

inline int
TBmpMenuItem::CompareText(const char* buffer) const
{
  return strcmp(text, buffer);
}

inline HMENU
TBmpMenuItem::GetHandle() const
{
  return hMenu;
}

inline int
TBmpMenuItem::GetImageIndex() const
{
  return imageIndex;
}

inline int
TBmpMenuItem::GetCommandId() const
{
  return commandId;
}

inline uint
TBmpMenuItem::GetPosition() const
{
  return position;
}

inline bool
TBmpMenuItem::operator ==(const TBmpMenuItem& other) const
{
  return (memcmp(this, &other, (size_t)&text - (size_t)&hMenu) == 0 &&
          strcmp(text, other.text) == 0);
}

inline void
TBmpMenuItem::SetHandle(HMENU handle)
{
  hMenu = handle;
}

inline void
TBmpMenuItem::SetImageIndex(int idx)
{
  imageIndex = idx;
}

inline void
TBmpMenuItem::SetCommandId(int idc)
{
  commandId = idc;
}

inline void
TBmpMenuItem::SetPosition(uint pos)
{
  position = pos;
}

// TBmpMenuImage inline implementation
//
inline
TBmpMenuImage::TBmpMenuImage()
:
 imageIndex(0), commandId(0)
{
}

inline
TBmpMenuImage::TBmpMenuImage(int idx, int idc)
:
 imageIndex(idx), commandId(idc)
{
}

inline int
TBmpMenuImage::GetImageIndex() const
{
  return imageIndex;
}

inline int
TBmpMenuImage::GetCommandId() const
{
  return commandId;
}

inline bool
TBmpMenuImage::operator ==(const TBmpMenuImage& other) const
{
  return (memcmp(this, &other, sizeof(TBmpMenuImage)));
}

inline void
TBmpMenuImage::SetImageIndex(int idx)
{
  imageIndex = idx;
}

inline void
TBmpMenuImage::SetCommandId(int idc)
{
  commandId = idc;
}

// TBmpMenu inline implementation
//
inline bool
TBmpMenu::DrawRecessed() const
{
  return bDrawRecessed;
}

inline bool
TBmpMenu::ShowAcclInTooltips() const
{
  return bShowAcclInTooltips;
}

inline bool
TBmpMenu::ShowLine() const
{
  return bShowLine;
}

inline bool
TBmpMenu::XPStyle() const
{
  return bXPStyle;
}

inline void
TBmpMenu::DrawRecessed(bool flag)
{
  bDrawRecessed = flag;
}

inline void
TBmpMenu::ShowAcclInTooltips(bool flag)
{
  bShowAcclInTooltips = flag;
}

inline void
TBmpMenu::ShowLine(bool flag)
{
  bShowLine = flag;
}

inline void
TBmpMenu::XPStyle(bool flag)
{
  bXPStyle = flag;
}

inline void
TBmpMenu::ExcludeBitmap(unsigned int resId)
{
  if (!excludedBitmaps.Find(resId))
    excludedBitmaps.Add(resId);
}

inline void
TBmpMenu::IncludeBitmap(unsigned int resId)
{
  excludedBitmaps.Detach(resId);
}

#endif  // TOOL_BMPMENU_H
